﻿using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.IO;
using Combinator;
using UIDemo;

namespace CombinatorTest
{
    [TestClass]
    public class TestCombinator
    {
        [TestMethod]
        public void TestTextWriterLogger()
        {
            MemoryStream stream = new MemoryStream();
            StreamWriter writer = new StreamWriter(stream);
            TextWriterLogger logger = new TextWriterLogger()
            {
                Writer = writer
            };

            logger.Write(0, new object[] { "Abcdefg" });
            logger.Write(0, new object[] { "Hijklmn" });
            logger.Write(0, new object[] { "Opqrst" });
            logger.Write(0, new object[] { "Uvwxyz" });

            stream.Seek(0, SeekOrigin.Begin);
            StreamReader reader = new StreamReader(stream);
            Assert.AreEqual("Abcdefg", reader.ReadLine());
            Assert.AreEqual("Hijklmn", reader.ReadLine());
            Assert.AreEqual("Opqrst", reader.ReadLine());
            Assert.AreEqual("Uvwxyz", reader.ReadLine());
            Assert.IsTrue(reader.EndOfStream);
        }

        [TestMethod]
        public void TestSequenceLogger()
        {
            MockLogger[] loggers = Enumerable.Range(0, 5).Select(i => new MockLogger()).ToArray();
            SequenceLogger sequenceLogger = new SequenceLogger()
            {
                Loggers = loggers
            };

            sequenceLogger.Write(0, new object[] { 1, 2, 3 });
            sequenceLogger.Write(0, new object[] { 4, 5, 6 });
            sequenceLogger.Write(0, new object[] { 7, 8, 9 });

            foreach (MockLogger logger in loggers)
            {
                Assert.AreEqual(3, logger.messages.Count);
                Assert.IsTrue(logger.messages[0].Cast<int>().SequenceEqual(new int[] { 1, 2, 3 }));
                Assert.IsTrue(logger.messages[1].Cast<int>().SequenceEqual(new int[] { 4, 5, 6 }));
                Assert.IsTrue(logger.messages[2].Cast<int>().SequenceEqual(new int[] { 7, 8, 9 }));
            }
        }

        [TestMethod]
        public void TestFilterLogger()
        {
            MockLogger logger = new MockLogger();
            FilterLogger filterLogger = new FilterLogger()
            {
                Logger = logger,
                Condition = i => i == 0
            };

            filterLogger.Write(0, new object[] { 1, 2, 3 });
            filterLogger.Write(1, new object[] { 4, 5, 6 });
            filterLogger.Write(0, new object[] { 7, 8, 9 });
            filterLogger.Write(2, new object[] { 10, 11, 12 });

            Assert.AreEqual(2, logger.messages.Count);
            Assert.IsTrue(logger.messages[0].Cast<int>().SequenceEqual(new int[] { 1, 2, 3 }));
            Assert.IsTrue(logger.messages[1].Cast<int>().SequenceEqual(new int[] { 7, 8, 9 }));
        }

        [TestMethod]
        public void TestLineNumberLogger()
        {
            MockLogger logger = new MockLogger();
            LineNumberLogger lineNumberLogger = new LineNumberLogger()
            {
                Logger = logger
            };

            lineNumberLogger.Write(0, new object[] { "ABC" });
            lineNumberLogger.Write(0, new object[] { "DEF" });
            lineNumberLogger.Write(0, new object[] { "GHI" });

            Assert.AreEqual(3, logger.messages.Count);
            Assert.IsTrue(logger.messages[0].Cast<string>().SequenceEqual(new string[] { "1: ", "ABC" }));
            Assert.IsTrue(logger.messages[1].Cast<string>().SequenceEqual(new string[] { "2: ", "DEF" }));
            Assert.IsTrue(logger.messages[2].Cast<string>().SequenceEqual(new string[] { "3: ", "GHI" }));
        }
    }

    class MockLogger : ILogger
    {
        public List<object[]> messages = new List<object[]>();

        #region ILogger Members

        public void Write(int level, object[] messages)
        {
            this.messages.Add(messages);
        }

        #endregion
    }
}
