﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using Combinator;
using InformationGenerator;
using System.Text.RegularExpressions;
using System.Threading;
using System.Reflection;

namespace UIDemo
{
    public partial class MainForm : Form, ILogger
    {
        public MainForm()
        {
            InitializeComponent();
            listViewErrors.GetType().GetProperty("DoubleBuffered", BindingFlags.NonPublic | BindingFlags.Instance).SetValue(listViewErrors, true, new object[] { });
        }

        private void buttonStart_Click(object sender, EventArgs e)
        {
            listViewErrors.Items.Clear();
            buttonStart.Enabled = false;
            new Thread(new ThreadStart(() =>
            {
                string path = typeof(Program).Assembly.Location;
                string folder = Path.GetDirectoryName(path);
                string logFile = folder + "\\buildchk.log";
                string warningFile = folder + "\\buildchk.wrn";
                string errorFile = folder + "\\buildchk.err";

                using (StreamWriter logWriter = new StreamWriter(logFile))
                using (StreamWriter warningWriter = new StreamWriter(warningFile))
                using (StreamWriter errorWriter = new StreamWriter(errorFile))
                {
                    ILogger logger = Logger.Sequence(
                        Logger.WriteTo(logWriter)
                            .WithTimestamp()
                            .WithExceptionDetail(printInnerException: true, printStackTrace: true),

                        Logger.WriteTo(warningWriter)
                            .WithLineNumber()
                            .AcceptLevel(Logger.Warning),

                        Logger.WriteTo(errorWriter)
                            .WithLineNumber()
                            .AcceptLevel(Logger.Error),

                        this
                            .AcceptLevel(Logger.Warning, Logger.Error)
                        );

                    LogGenerator.Generate(logger);
                }

                this.Invoke(new MethodInvoker(() =>
                {
                    buttonStart.Enabled = true;
                    MessageBox.Show("Finished!", this.Text);
                }));
            })).Start();
        }

        public void Write(int level, object[] messages)
        {
            string line = messages[0].ToString();
            string pattern = level == Logger.Warning ? ": warning" : ": error";
            int index = line.LastIndexOf(pattern);

            string fileInfo = line.Substring(0, index);
            string message = line.Substring(index + 2);
            string fileName = "";
            string column = "";

            index = fileInfo.LastIndexOf('(');
            if (index != -1)
            {
                fileName = Path.GetFileName(fileInfo.Substring(0, index));
                column = fileInfo.Substring(index + 1, fileInfo.LastIndexOf(')') - index - 1);
            }

            this.Invoke(new MethodInvoker(() =>
            {
                ListViewItem item = new ListViewItem("");
                item.ImageIndex = level == Logger.Error ? 0 : 1;
                item.SubItems.Add((listViewErrors.Items.Count + 1).ToString());
                item.SubItems.Add(message);
                item.SubItems.Add(fileName);
                item.SubItems.Add(column);
                listViewErrors.Items.Add(item);
                listViewErrors.EnsureVisible(listViewErrors.Items.Count - 1);
            }));
        }
    }
}
